@@ -23,10 +23,6 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #define _GNU_SOURCE
 #include <stdlib.h>
 #include <string.h>
@@ -56,7 +52,6 @@
 #include "xfce-desktop-utils.h"
 
 #define GPM_COMMAND "xfce4-power-manager-settings"
-#define CONFIGURE_COMMAND "xfce4-screensaver-configure.py"
 
 enum {
     NAME_COLUMN = 0,
@@ -564,22 +559,6 @@ config_set_logout_command (const gchar *command) {
     xfconf_channel_set_string (screensaver_channel, KEY_LOGOUT_COMMAND, command);
 }
 
-static gchar *
-config_get_theme_arguments (const gchar *theme) {
-    gchar *property;
-    gchar *arguments;
-    gchar *theme_name;
-
-    theme_name = g_utf8_substring (theme, 13, g_utf8_strlen (theme, -1));
-    property = g_strdup_printf ("/screensavers/%s/arguments", theme_name);
-    arguments = xfconf_channel_get_string (screensaver_channel, property, "");
-
-    g_free (theme_name);
-    g_free (property);
-
-    return arguments;
-}
-
 static void
 job_set_theme (GSJob *local_job,
                const char *theme) {
@@ -591,7 +570,7 @@ job_set_theme (GSJob *local_job,
 
     info = gs_theme_manager_lookup_theme_info (theme_manager, theme);
     if (info != NULL) {
-        arguments = config_get_theme_arguments (theme);
+        arguments = get_theme_arguments (screensaver_channel, theme);
         command = g_strdup_printf ("%s %s", gs_theme_info_get_exec (info), arguments);
     }
 
@@ -1765,7 +1744,6 @@ configure_capplet (void) {
     GtkWidget *root_warning_infobar;
     GtkWidget *preview_button;
     GtkWidget *gpm_button;
-    GtkWidget *configure_toolbar;
     GtkWidget *configure_button;
     GtkWidget *fullscreen_preview_window;
     GtkWidget *fullscreen_preview_area;
@@ -1811,7 +1789,6 @@ configure_capplet (void) {
     preview_button = GTK_WIDGET (gtk_builder_get_object (builder, "preview_button"));
     gpm_button = GTK_WIDGET (gtk_builder_get_object (builder, "power_management_button"));
     configure_button = GTK_WIDGET (gtk_builder_get_object (builder, "configure_button"));
-    configure_toolbar = GTK_WIDGET (gtk_builder_get_object (builder, "configure_toolbar"));
     fullscreen_preview_window = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_preview_window"));
     fullscreen_preview_area = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_preview_area"));
     fullscreen_preview_close = GTK_WIDGET (gtk_builder_get_object (builder, "fullscreen_preview_close"));
@@ -1847,6 +1824,10 @@ configure_capplet (void) {
 
     gtk_widget_set_no_show_all (root_warning_infobar, TRUE);
 
+    /* necessary in the prefs dialog for some reason, see
+     * https://gitlab.xfce.org/apps/xfce4-screensaver/-/issues/183 */
+    gtk_widget_set_visual (preview, gdk_visual_get_best ());
+
     if (!is_program_in_path (GPM_COMMAND)) {
         gtk_widget_set_no_show_all (gpm_button, TRUE);
         gtk_widget_hide (gpm_button);
@@ -1858,15 +1839,10 @@ configure_capplet (void) {
                       G_CALLBACK (key_changed_cb),
                       NULL);
 
-    if (!is_program_in_path (CONFIGURE_COMMAND)) {
-        gtk_widget_set_no_show_all (configure_toolbar, TRUE);
-        gtk_widget_hide (configure_toolbar);
-    } else {
-        g_signal_connect (configure_button,
-                          "clicked",
-                          G_CALLBACK (configure_button_clicked_cb),
-                          screensaver_channel);
-    }
+    g_signal_connect (configure_button,
+                      "clicked",
+                      G_CALLBACK (configure_button_clicked_cb),
+                      screensaver_channel);
 
     /* Idle delay */
     widget = GTK_WIDGET (gtk_builder_get_object (builder, "saver_idle_activation_delay"));
